'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import logging
import sys

from clazzes.gitlabclient import glclogging
from clazzes.gitlabclient.dto import GLCConfiguration
from clazzes.gitlabclient.restclient import GLCRestClient

log = glclogging.getLogger()
glcConfiguration = None


def describeUsage(argv0):
    usage = ""
    usage += "\n"
    usage += "Usage: %s [options...]\n"%argv0
    usage += "\n"
    usage += "Checks if the Gitlab v4 APIs of one or all configured instances are reachable.\n"
    usage += glclogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -a             Process all Gitlab instances configured in gitlab-client.cnf\n"
    usage += "    -n gitlabName  Use Gitlab instance with this section name in gitlab-client.cnf\n"
    return usage


def run_glc_ping(argv0,args):
    
    try:
        opts, garbage = getopt.getopt(args, "adhn:qtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    glcConfiguration = GLCConfiguration()
    restclient = GLCRestClient(glcConfiguration)
    
    doAllGitlabs = False
    gitlabName = ""
            
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ("-a"):
            doAllGitlabs = True
        elif o in ("-n"):
            gitlabName = a
        else:
            log.warning("Unhandled option: %s %s" % (o, a))
    if len(garbage) > 0:
        log.warning("Unhandled options: %s" % " ".join(garbage))

    # either all Gitlabs ...
    if doAllGitlabs:
        log.debug("Using all known Gitlab instances")
        for gitlabInfo in glcConfiguration.gitlabInfos.values():
            restclient.pingGitlab(gitlabInfo=gitlabInfo)
        sys.exit(0)
    
    # ... or a single Gitlab    
    if len(gitlabName) == 0:
        gitlabName = glcConfiguration.primaryGitlabName
        log.debug("Using primary Gitlab instance, %s" % gitlabName)

    gitlabInfo = glcConfiguration.gitlabInfos.get(gitlabName)
    
    if gitlabInfo == None:
        log.error("Unkown Gitlab instance name: '%s'" %
              gitlabName)
        sys.exit(1)
    restclient.pingGitlab(gitlabInfo=gitlabInfo)

    sys.exit(0)


