'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import logging
import sys

from clazzes.gitlabclient import glclogging
from clazzes.gitlabclient.dto import GLCConfiguration
from clazzes.gitlabclient.restclient import GLCRestClient

log = glclogging.getLogger()
glcConfiguration = None


def describeUsage(argv0):
    usage = ""
    usage += "\n"
    usage += "Usage: %s [options...] [projectFilter]\n"%argv0
    usage += "\n"
    usage += "Lists all projects matching the filter.\n"
    usage += "\n"
    usage += "An eventual projectFilter is applied to the projects' path_with_namespace\n"
    usage += glclogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -n gitlabName  Use Gitlab instance with this section name in gitlab-client.cnf\n"
    return usage


def run_glc_projects_list(argv0,args):
    
    try:
        opts, projectFilters = getopt.getopt(args, "dhn:qtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    glcConfiguration = GLCConfiguration()
    restclient = GLCRestClient(glcConfiguration)
    
    gitlabName = ""
    projectFilter = "";
            
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ("-n"):
            gitlabName = a
        else:
            log.warning("Unhandled option: %s %s" % (o, a))
            
    if len(projectFilters) == 0:
        log.debug("No projectFilter specified, going to list really all.")
    elif len(projectFilters) == 1:
        projectFilter = projectFilters[0]
    else:
        log.error("Multiple projectFilters specified, this is not supported.")
        sys.exit(1)
    
    # single Gitlab only    
    if len(gitlabName) == 0:
        gitlabName = glcConfiguration.primaryGitlabName
        log.debug("Using primary Gitlab instance, %s" % gitlabName)

    gitlabInfo = glcConfiguration.gitlabInfos.get(gitlabName)
    
    if gitlabInfo == None:
        log.error("Unkown Gitlab instance name: '%s'" %
              gitlabName)
        sys.exit(1)
        
    restclient.fetchProjectList(gitlabInfo=gitlabInfo, projectFilter=projectFilter)

    if len(projectFilter) == 0:
        log.p("\nGitlab %s has %d projects accessible by you:\n" % (
            gitlabName, len(gitlabInfo.projectInfos)))
    else:
        log.p("\nGitlab %s has %d projects accessible by you matching the specified filter:\n" % (
            gitlabName, len(gitlabInfo.projectInfos)))
    for projectPath in sorted(gitlabInfo.projectInfos.keys()):
        projectInfo = gitlabInfo.projectInfos[projectPath]
        log.p(projectInfo.describeValues(indent="  "))
        
    sys.exit(0)
