'''
  Copyleft ITEG IT-Engineers GmbH 2021
'''

#import fileinput
#import ipaddress
#import json
#import os
#import re
#import subprocess
import sys

from datetime import datetime

#from clazzes.tools.logtools import configureStderr


def getLogger():
    return Logger.glcLogger

def initLogger(args=[]):
    Logger.glcLogger = Logger()
    Logger.glcLogger.parseArgs(args=args)
    
class Logger:
    glcLogger = None
    pyLogger = None
    
    def __init__(self):
        self.verbosity = 1
        self.prefixTimestamp = False
        
    def parseArgs(self, args=None):
        self.verbosity = 1;
        #print("Logger.parseOpts(): start self.verbosity=%d" % self.verbosity, file=sys.stdout, flush=True)
        #debugModules = None
        if args != None:
            for o in args:
                if o in ["-d"]:
                    if self.verbosity < 3:
                        self.verbosity = 3
                    #debugModules = ["iteg.lxcutils"]
                elif o in ["-q"]:
                    self.verbosity = 0
                elif o in ["-t"]:
                    self.prefixTimestamp = True
                elif o in ["-v"]:
                    self.verbosity += 1
        #print("Logger.parseOpts(): post-parsing self.verbosity=%d" % self.verbosity, file=sys.stdout, flush=True)
        '''
        # this breaks some situations but should be OK for glcutils:
        from clazzes.tools.logtools import configureStderr
        configureStderr(debugNames=debugModules)
        pyLogger = logging.getLogger(__name__)
        '''

    def getPrefix(self):
        if self.prefixTimestamp:
            return datetime.now().isoformat(sep=" ", timespec="milliseconds") + ": "
        else:
            return ""
            
    def debug(self, m:str=""):
        if self.verbosity < 3:
            return
        if len(m) == 0:
            return 
        print(self.getPrefix() + "DEBUG: " + m, file=sys.stdout, flush=True)
        
    def info(self, m:str=""):
        if self.verbosity < 2:
            return
        if len(m) == 0:
            return 
        print(self.getPrefix() + "INFO: " + m, file=sys.stdout, flush=True)
    
    def notice(self, m:str=""):
        if self.verbosity < 1:
            return
        if len(m) == 0:
            return 
        print(self.getPrefix() + "NOTICE: " + m, file=sys.stdout, flush=True)
        
    def good(self, m:str=""):
        if self.verbosity < 1:
            return
        if len(m) == 0:
            return 
        print(self.getPrefix() + "GOOD: " + m, file=sys.stdout, flush=True)
        
    def warning(self, m:str=""):
        if self.verbosity < 1:
            return
        print(self.getPrefix() + "WARNING: " + m, file=sys.stderr, flush=True)
        
    def error(self, m:str=""):
        print(self.getPrefix() + "ERROR: " + m, file=sys.stderr, flush=True)

    def p(self, m:str=""):
        # NO timestamp prefix here!
        print(m, file=sys.stdout, flush=True)


def getCommonUsageOptions():
    usage = "\n"
    usage += "  Common options are:\n"
    usage += "    --help|-h Print this help text.\n"
    usage += "    -d        Activate debug logging.\n"
    usage += "    -q        Lower verbosity to minimal value 0.\n"
    usage += "    -t        Prefix log lines with a timestamp.\n"
    usage += "    -v        Raise verbosity by 1.\n"
    usage += "\n"
    return usage
