'''
  Copyleft ITEG IT-Engineers GmbH 2022
'''

#import base64
#import configparser
import io
import json
#import logging
#import os
import re
#import subprocess
import sys

#from clazzes.datetools import import to_datetime_timestamp_millis,
#from datetime import datetime
#from http.client import HTTPSConnection
from urllib.error import HTTPError
#from urllib.parse import quote_plus
#, urlencode
from urllib.request import Request, urlopen

from clazzes.gitlabclient import dto
from clazzes.gitlabclient import glclogging
from clazzes.gitlabclient import helpers


log = glclogging.getLogger()


class GLCRestClient:
    '''
      the REST client doing the HTTPS work
      
      API documentation is at https://docs.gitlab.com/ee/api/
      
      Maybe we can build upon their OpenAPI definition?
    '''

    def __init__(self, glcConfiguration: dto.GLCConfiguration):
        self.glcConfiguration = glcConfiguration 


    def pingGitlab(self, gitlabInfo: dto.GitlabInfo):
        if gitlabInfo == None:
            raise Exception("ERROR: pingGitlab() called with gitlabInfo None")
        req = Request(gitlabInfo.api_url + "/version")
        #log.debug("pingGitlab() using this auth header: '%s'" % gitlabInfo.api_auth_headervalue)
        req.add_header(gitlabInfo.api_auth_headername, gitlabInfo.api_auth_headervalue)
        with urlopen(req) as responseConnection:
            response = responseConnection.read()
            parsedResponse = json.loads(response)
        #jsonFn = "/tmp/glc_version.json"
        #json.dump(parsedResponse, io.open(jsonFn,"w"), indent=2) # wb
        #log.debug("Response saved to %s" % jsonFn)
        gitlabSystemInfo = dto.GitlabSystemInfo(sysinfo=parsedResponse)
        gitlabInfo.glc_systeminfo = gitlabSystemInfo
        log.good("Successfully connected to Gitlab %s:\n%s" % (
            gitlabInfo.name, gitlabSystemInfo.describeValues(indent="  ")))

    def fetchProjectList(self, gitlabInfo: dto.GitlabInfo, projectFilter:str = ""):
        if gitlabInfo == None:
            raise Exception("ERROR: pingGitlab() called with gitlabInfo None")
        next_page_url = gitlabInfo.api_url + "/projects?per_page=100&simple=true"
        # filter?
        reFilter = None
        if len(projectFilter) > 0:
            reFilter = re.compile(projectFilter)
        # reset holder
        gitlabInfo.projectInfos = {}
        nrRetrievedProjects = 0
        nrRequest = 0
        #
        reNextPageUrlFromLinks = re.compile('.*<([^>]+)>; rel="next".*')
        reProjectIsInstanceMonitoring = re.compile("^gitlab-instance-.*/Monitoring$")
        while len(next_page_url) > 0:
            nrRequest += 1
            req = Request(next_page_url)
            #log.debug("fetchProjectList() using this auth header: '%s'" % gitlabInfo.api_auth_headervalue)
            req.add_header(gitlabInfo.api_auth_headername, gitlabInfo.api_auth_headervalue)
            with urlopen(req) as responseConnection:
                responseInfo = responseConnection.info()
                # read heaer 'link'                    
                #log.debug("responseInfo: %s" % responseInfo)
                nextLinksString = responseInfo.get(name="link")
                if nextLinksString == None:
                    log.debug("No 'link' header in response, done after this block")
                    next_page_url = ""
                elif len(nextLinksString) == 0:
                    log.debug("Empty 'link' header in response, done after this block")
                    next_page_url = ""
                else:
                    log.debug("'link' header in response not empty: %s" % nextLinksString)
                    matchNextPageUrlFromLinks = reNextPageUrlFromLinks.fullmatch(nextLinksString)
                    if matchNextPageUrlFromLinks == None:
                        log.debug("'link' header does not provide next link, done after this block")
                        next_page_url = ""
                    else:
                        next_page_url = matchNextPageUrlFromLinks.group(1)
                        log.debug("'link' header provided next_page_url: %s" % next_page_url)
                # read data                    
                response = responseConnection.read()
                parsedResponse = json.loads(response)
            #jsonFn = "/tmp/glc_projects_%d.json" % nrRequest
            #json.dump(parsedResponse, io.open(jsonFn,"w"), indent=2) # wb
            #log.debug("Response saved to %s" % jsonFn)
            # walk
            for projectDict in parsedResponse:
                projectInfo = dto.GitlabProjectInfo(projectDict=projectDict)
                nrRetrievedProjects += 1
                if reFilter != None:
                    if reFilter.search(projectInfo.path_with_namespace) == None:
                        log.debug("fetchProjectList(): '%s' does NOT match '%s', skipping" % (projectFilter, projectInfo.path_with_namespace))
                        continue
                    log.debug("fetchProjectList(): '%s' does MATCH '%s', using it" % (projectFilter, projectInfo.path_with_namespace))
                if reProjectIsInstanceMonitoring.match(projectInfo.path_with_namespace):
                    log.debug("fetchProjectList(): '%s' is internal Monitoring project, skipping" % projectInfo.path_with_namespace)
                    continue
                log.debug("fetchProjectList(): Remembering project '%s'" % projectInfo.path_with_namespace)
                gitlabInfo.projectInfos[projectInfo.path_with_namespace] = projectInfo
            if reFilter == None:
                log.good("After round %d successfully retrieved %d project infos." % 
                         (nrRequest, len(gitlabInfo.projectInfos)))
            else:
                log.good("After round %d successfully retrieved %d project infos, %d matching the filter." % (
                    nrRequest, nrRetrievedProjects, len(gitlabInfo.projectInfos)))
